@@ -1,8 +1,8 @@
 //-----------------------------------------------------------------------------
 //
 // ImageLib Sources
-// Copyright (C) 2000-2008 by Denton Woods
-// Last modified: 12/06/2006
+// Copyright (C) 2000-2009 by Denton Woods
+// Last modified: 03/07/2009
 //
 // Filename: src-IL/src/il_convert.c
 //
@@ -33,7 +33,7 @@ ILimage *iConvertPalette(ILimage *Image, ILenum DestFormat)
 
 	ilCopyImageAttr(NewImage, Image);
 
-	if (!iCurImage->Pal.Palette || !iCurImage->Pal.PalSize || iCurImage->Pal.PalType == IL_PAL_NONE || iCurImage->Bpp != 1) {
+	if (!Image->Pal.Palette || !Image->Pal.PalSize || Image->Pal.PalType == IL_PAL_NONE || Image->Bpp != 1) {
 		ilCloseImage(NewImage);
 		ilSetError(IL_ILLEGAL_OPERATION);
 		return NULL;
@@ -270,7 +270,7 @@ alloc_error:
 // In il_quantizer.c
 ILimage *iQuantizeImage(ILimage *Image, ILuint NumCols);
 // In il_neuquant.c
-ILimage *iNeuQuant(ILimage *Image);
+ILimage *iNeuQuant(ILimage *Image, ILuint NumCols);
 
 // Converts an image from one format to another
 ILAPI ILimage* ILAPIENTRY iConvertImage(ILimage *Image, ILenum DestFormat, ILenum DestType)
@@ -295,13 +295,13 @@ ILAPI ILimage* ILAPIENTRY iConvertImage(ILimage *Image, ILenum DestFormat, ILenu
 		NewImage = iConvertPalette(Image, DestFormat);
 
 		//added test 2003-09-01
-		if(NewImage == NULL)
+		if (NewImage == NULL)
 			return NULL;
 
 		if (DestType == NewImage->Type)
 			return NewImage;
 
-		NewData = (ILubyte*)ilConvertBuffer(NewImage->SizeOfData, NewImage->Format, DestFormat, NewImage->Type, DestType, NewImage->Data);
+		NewData = (ILubyte*)ilConvertBuffer(NewImage->SizeOfData, NewImage->Format, DestFormat, NewImage->Type, DestType, NULL, NewImage->Data);
 		if (NewData == NULL) {
 			ifree(NewImage);  // ilCloseImage not needed.
 			return NULL;
@@ -320,9 +320,9 @@ ILAPI ILimage* ILAPIENTRY iConvertImage(ILimage *Image, ILenum DestFormat, ILenu
 	}
 	else if (DestFormat == IL_COLOUR_INDEX && Image->Format != IL_LUMINANCE) {
 		if (iGetInt(IL_QUANTIZATION_MODE) == IL_NEU_QUANT)
-			return iNeuQuant(Image);
+			return iNeuQuant(Image, iGetInt(IL_MAX_QUANT_INDICES));
 		else // Assume IL_WU_QUANT otherwise.
-			return iQuantizeImage(Image, iGetInt(IL_MAX_QUANT_INDEXS));
+			return iQuantizeImage(Image, iGetInt(IL_MAX_QUANT_INDICES));
 	}
 	else {
 		NewImage = (ILimage*)icalloc(1, sizeof(ILimage));  // Much better to have it all set to 0.
@@ -362,7 +362,7 @@ ILAPI ILimage* ILAPIENTRY iConvertImage(ILimage *Image, ILenum DestFormat, ILenu
 			memcpy(NewImage->Data, Image->Data, Image->SizeOfData);
 		}
 		else {
-			NewImage->Data = (ILubyte*)ilConvertBuffer(Image->SizeOfData, Image->Format, DestFormat, Image->Type, DestType, Image->Data);
+			NewImage->Data = (ILubyte*)ilConvertBuffer(Image->SizeOfData, Image->Format, DestFormat, Image->Type, DestType, NULL, Image->Data);
 			if (NewImage->Data == NULL) {
 				ifree(NewImage);  // ilCloseImage not needed.
 				return NULL;
@@ -375,9 +375,15 @@ ILAPI ILimage* ILAPIENTRY iConvertImage(ILimage *Image, ILenum DestFormat, ILenu
 
 
 //! Converts the current image to the DestFormat format.
+/*! \param DestFormat An enum of the desired output format.  Any format values are accepted.
+    \param DestType An enum of the desired output type.  Any type values are accepted.
+	\exception IL_ILLEGAL_OPERATION No currently bound image
+	\exception IL_INVALID_CONVERSION DestFormat or DestType was an invalid identifier.
+	\exception IL_OUT_OF_MEMORY Could not allocate enough memory.
+	\return Boolean value of failure or success*/
 ILboolean ILAPIENTRY ilConvertImage(ILenum DestFormat, ILenum DestType)
 {
-	ILimage *Image, *pCurrImage;
+	ILimage *Image, *pCurImage;
 
 	if (iCurImage == NULL) {
 		ilSetError(IL_ILLEGAL_OPERATION);
@@ -398,36 +404,36 @@ ILboolean ILAPIENTRY ilConvertImage(ILenum DestFormat, ILenum DestType)
 		ilAddAlphaKey(iCurImage);
 	}
 
-	pCurrImage = iCurImage;
-	while (pCurrImage != NULL)
+	pCurImage = iCurImage;
+	while (pCurImage != NULL)
 	{
-		Image = iConvertImage(pCurrImage, DestFormat, DestType);
+		Image = iConvertImage(pCurImage, DestFormat, DestType);
 		if (Image == NULL)
 			return IL_FALSE;
 
-		//ilCopyImageAttr(pCurrImage, Image);  // Destroys subimages.
+		//ilCopyImageAttr(pCurImage, Image);  // Destroys subimages.
 
 		// We don't copy the colour profile here, since it stays the same.
 		//	Same with the DXTC data.
-		pCurrImage->Format = DestFormat;
-		pCurrImage->Type = DestType;
-		pCurrImage->Bpc = ilGetBpcType(DestType);
-		pCurrImage->Bpp = ilGetBppFormat(DestFormat);
-		pCurrImage->Bps = pCurrImage->Width * pCurrImage->Bpc * pCurrImage->Bpp;
-		pCurrImage->SizeOfPlane = pCurrImage->Bps * pCurrImage->Height;
-		pCurrImage->SizeOfData = pCurrImage->Depth * pCurrImage->SizeOfPlane;
-		if (pCurrImage->Pal.Palette && pCurrImage->Pal.PalSize && pCurrImage->Pal.PalType != IL_PAL_NONE)
-			ifree(pCurrImage->Pal.Palette);
-		pCurrImage->Pal.Palette = Image->Pal.Palette;
-		pCurrImage->Pal.PalSize = Image->Pal.PalSize;
-		pCurrImage->Pal.PalType = Image->Pal.PalType;
+		pCurImage->Format = DestFormat;
+		pCurImage->Type = DestType;
+		pCurImage->Bpc = ilGetBpcType(DestType);
+		pCurImage->Bpp = ilGetBppFormat(DestFormat);
+		pCurImage->Bps = pCurImage->Width * pCurImage->Bpc * pCurImage->Bpp;
+		pCurImage->SizeOfPlane = pCurImage->Bps * pCurImage->Height;
+		pCurImage->SizeOfData = pCurImage->Depth * pCurImage->SizeOfPlane;
+		if (pCurImage->Pal.Palette && pCurImage->Pal.PalSize && pCurImage->Pal.PalType != IL_PAL_NONE)
+			ifree(pCurImage->Pal.Palette);
+		pCurImage->Pal.Palette = Image->Pal.Palette;
+		pCurImage->Pal.PalSize = Image->Pal.PalSize;
+		pCurImage->Pal.PalType = Image->Pal.PalType;
 		Image->Pal.Palette = NULL;
-		ifree(pCurrImage->Data);
-		pCurrImage->Data = Image->Data;
+		ifree(pCurImage->Data);
+		pCurImage->Data = Image->Data;
 		Image->Data = NULL;
 		ilCloseImage(Image);
 
-		pCurrImage = pCurrImage->Next;
+		pCurImage = pCurImage->Next;
 	}
 
 	return IL_TRUE;
@@ -1068,36 +1074,49 @@ be loaded anyway. Thanks to Chris Lux for pointing this out.
 
 ILboolean ilFixImage()
 {
+	ILuint NumFaces,  f;
 	ILuint NumImages, i;
 	ILuint NumMipmaps,j;
 	ILuint NumLayers, k;
 
-	NumImages  = ilGetInteger(IL_NUM_IMAGES);
+	NumImages = ilGetInteger(IL_NUM_IMAGES);
 	for (i = 0; i <= NumImages; i++) {
 		ilBindImage(ilGetCurName());  // Set to parent image first.
 		if (!ilActiveImage(i))
 			return IL_FALSE;
 
-		NumLayers = ilGetInteger(IL_NUM_LAYERS);
-		for (k = 0; k <= NumLayers; k++) {
+		NumFaces = ilGetInteger(IL_NUM_FACES);
+		for (f = 0; f <= NumFaces; f++) {
 			ilBindImage(ilGetCurName());  // Set to parent image first.
 			if (!ilActiveImage(i))
 				return IL_FALSE;
-			if (!ilActiveLayer(k))
+			if (!ilActiveFace(f))
 				return IL_FALSE;
 
-			NumMipmaps = ilGetInteger(IL_NUM_MIPMAPS);
-
-			for (j = 0; j <= NumMipmaps; j++) {
-				ilBindImage(ilGetCurName());	// Set to parent image first.
+			NumLayers = ilGetInteger(IL_NUM_LAYERS);
+			for (k = 0; k <= NumLayers; k++) {
+				ilBindImage(ilGetCurName());  // Set to parent image first.
 				if (!ilActiveImage(i))
 					return IL_FALSE;
+				if (!ilActiveFace(f))
+					return IL_FALSE;
 				if (!ilActiveLayer(k))
 					return IL_FALSE;
-				if (!ilActiveMipmap(j))
-					return IL_FALSE;
-				if (!ilFixCur())
-					return IL_FALSE;
+
+				NumMipmaps = ilGetInteger(IL_NUM_MIPMAPS);
+				for (j = 0; j <= NumMipmaps; j++) {
+					ilBindImage(ilGetCurName());	// Set to parent image first.
+					if (!ilActiveImage(i))
+						return IL_FALSE;
+					if (!ilActiveFace(f))
+						return IL_FALSE;
+					if (!ilActiveLayer(k))
+						return IL_FALSE;
+					if (!ilActiveMipmap(j))
+						return IL_FALSE;
+					if (!ilFixCur())
+						return IL_FALSE;
+				}
 			}
 		}
 	}
