@@ -1,8 +1,8 @@
 //-----------------------------------------------------------------------------
 //
 // ImageLib Sources
-// Copyright (C) 2000-2008 by Denton Woods
-// Last modified: 08/17/2008
+// Copyright (C) 2000-2009 by Denton Woods
+// Last modified: 02/01/2009
 //
 // Filename: src-IL/src/il_devil.c
 //
@@ -16,6 +16,7 @@
 #include <limits.h>
 #include "il_manip.h"
 
+
 ILAPI ILboolean ILAPIENTRY ilInitImage(ILimage *Image, ILuint Width, ILuint Height, ILuint Depth, ILubyte Bpp, ILenum Format, ILenum Type, void *Data)
 {
 	ILubyte BpcType = ilGetBpcType(Type);
@@ -24,12 +25,12 @@ ILAPI ILboolean ILAPIENTRY ilInitImage(ILimage *Image, ILuint Width, ILuint Heig
 		return IL_FALSE;
 	}
 
-	memset (Image, 0, sizeof(ILimage));
+	memset(Image, 0, sizeof(ILimage));
 
 	////
-	if( Width  == 0 ) Width = 1;
-	if( Height == 0 ) Height = 1;
-	if( Depth  == 0 ) Depth = 1;
+	if (Width  == 0) Width = 1;
+	if (Height == 0) Height = 1;
+	if (Depth  == 0) Depth = 1;
 	Image->Width	   = Width;
 	Image->Height	   = Height;
 	Image->Depth	   = Depth;
@@ -73,7 +74,7 @@ ILAPI ILimage* ILAPIENTRY ilNewImage(ILuint Width, ILuint Height, ILuint Depth,
 		return NULL;
 	}
 
-	if (!ilInitImage (Image, Width, Height, Depth, Bpp, ilGetFormatBpp(Bpp), ilGetTypeBpc(Bpc), NULL)) {
+	if (!ilInitImage(Image, Width, Height, Depth, Bpp, ilGetFormatBpp(Bpp), ilGetTypeBpc(Bpc), NULL)) {
 		if (Image->Data != NULL) {
 			ifree(Image->Data);
 		}
@@ -84,6 +85,8 @@ ILAPI ILimage* ILAPIENTRY ilNewImage(ILuint Width, ILuint Height, ILuint Depth,
 	return Image;
 }
 
+
+// Same as above but allows specification of Format and Type
 ILAPI ILimage* ILAPIENTRY ilNewImageFull(ILuint Width, ILuint Height, ILuint Depth, ILubyte Bpp, ILenum Format, ILenum Type, void *Data)
 {
 	ILimage *Image;
@@ -97,7 +100,7 @@ ILAPI ILimage* ILAPIENTRY ilNewImageFull(ILuint Width, ILuint Height, ILuint Dep
 		return NULL;
 	}
 
-	if (!ilInitImage (Image, Width, Height, Depth, Bpp, Format, Type, Data)) {
+	if (!ilInitImage(Image, Width, Height, Depth, Bpp, Format, Type, Data)) {
 		if (Image->Data != NULL) {
 			ifree(Image->Data);
 		}
@@ -110,6 +113,17 @@ ILAPI ILimage* ILAPIENTRY ilNewImageFull(ILuint Width, ILuint Height, ILuint Dep
 
 
 //! Changes the current bound image to use these new dimensions (current data is destroyed).
+/*! \param Width Specifies the new image width.  This cannot be 0.
+	\param Height Specifies the new image height.  This cannot be 0.
+	\param Depth Specifies the new image depth.  This cannot be 0.
+	\param Bpp Number of channels (ex. 3 for RGB)
+	\param Format Enum of the desired format.  Any format values are accepted.
+	\param Type Enum of the desired type.  Any type values are accepted.
+	\param Data Specifies data that should be copied to the new image. If this parameter is NULL, no data is copied, and the new image data consists of undefined values.
+	\exception IL_ILLEGAL_OPERATION No currently bound image.
+	\exception IL_INVALID_PARAM One of the parameters is incorrect, such as one of the dimensions being 0.
+	\exception IL_OUT_OF_MEMORY Could not allocate enough memory.
+	\return Boolean value of failure or success*/
 ILboolean ILAPIENTRY ilTexImage(ILuint Width, ILuint Height, ILuint Depth, ILubyte Bpp, ILenum Format, ILenum Type, void *Data)
 {
 	return ilTexImage_(iCurImage, Width, Height, Depth, Bpp, Format, Type, Data);
@@ -133,6 +147,7 @@ ILAPI ILboolean ILAPIENTRY ilTexImage_(ILimage *Image, ILuint Width, ILuint Heig
 
 	ilCloseImage(Image->Mipmaps);
 	ilCloseImage(Image->Next);
+	ilCloseImage(Image->Faces);
 	ilCloseImage(Image->Layers);
 
 	if (Image->AnimList) ifree(Image->AnimList);
@@ -142,7 +157,7 @@ ILAPI ILboolean ILAPIENTRY ilTexImage_(ILimage *Image, ILuint Width, ILuint Heig
 
 	////
 
-	// Also check against format?
+	//@TODO: Also check against format?
 	/*if (Width == 0 || Height == 0 || Depth == 0 || Bpp == 0) {
 		ilSetError(IL_INVALID_PARAM);
 	return IL_FALSE;
@@ -153,6 +168,11 @@ ILAPI ILboolean ILAPIENTRY ilTexImage_(ILimage *Image, ILuint Width, ILuint Heig
 
 
 //! Uploads Data of the same size to replace the current image's data.
+/*! \param Data New image data to update the currently bound image
+	\exception IL_ILLEGAL_OPERATION No currently bound image
+	\exception IL_INVALID_PARAM Data was NULL.
+	\return Boolean value of failure or success
+*/
 ILboolean ILAPIENTRY ilSetData(void *Data)
 {
 	if (iCurImage == NULL) {
@@ -181,7 +201,14 @@ ILAPI ILboolean ILAPIENTRY ilTexSubImage_(ILimage *Image, void *Data)
 }
 
 
-ILubyte* ILAPIENTRY ilGetData()
+//! Returns a pointer to the current image's data.
+/*! The pointer to the image data returned by this function is only valid until any
+    operations are done on the image.  After any operations, this function should be
+	called again.  The pointer can be cast to other types for images that have more
+	than one byte per channel for easier access to data.
+	\exception IL_ILLEGAL_OPERATION No currently bound image
+	\return ILubyte pointer to image data.*/
+ILubyte* ILAPIENTRY ilGetData(void)
 {
 	if (iCurImage == NULL) {
 		ilSetError(IL_ILLEGAL_OPERATION);
@@ -192,7 +219,13 @@ ILubyte* ILAPIENTRY ilGetData()
 }
 
 
-ILubyte* ILAPIENTRY ilGetPalette()
+//! Returns a pointer to the current image's palette data.
+/*! The pointer to the image palette data returned by this function is only valid until
+	any operations are done on the image.  After any operations, this function should be
+	called again.
+	\exception IL_ILLEGAL_OPERATION No currently bound image
+	\return ILubyte pointer to image palette data.*/
+ILubyte* ILAPIENTRY ilGetPalette(void)
 {
 	if (iCurImage == NULL) {
 		ilSetError(IL_ILLEGAL_OPERATION);
@@ -636,14 +669,14 @@ ILboolean ILAPIENTRY ilBlit(ILuint Source, ILint DestX,  ILint DestY,   ILint De
 	ILfloat		Back;
 
 	// Check if the desiination image really exists
-	if( DestName == 0 || iCurImage == NULL ) {
+	if (DestName == 0 || iCurImage == NULL) {
 		ilSetError(IL_ILLEGAL_OPERATION);
 		return IL_FALSE;
 	}
 	Dest = iCurImage;
 	
 	// set the destination image to upper left origin
-	if( Dest->Origin == IL_ORIGIN_LOWER_LEFT ) {  // Dest
+	if (Dest->Origin == IL_ORIGIN_LOWER_LEFT) {  // Dest
 		DestFlipped = IL_TRUE;
 		ilFlipImage();
 	}
@@ -651,7 +684,7 @@ ILboolean ILAPIENTRY ilBlit(ILuint Source, ILint DestX,  ILint DestY,   ILint De
 	ilBindImage(Source);
 	
 	// Check if the source image really exists
-	if( iCurImage == NULL ) {
+	if (iCurImage == NULL) {
 		ilSetError(IL_INVALID_PARAM);
 		return IL_FALSE;
 	}
@@ -678,7 +711,7 @@ ILboolean ILAPIENTRY ilBlit(ILuint Source, ILint DestX,  ILint DestY,   ILint De
 	}
 	
 	// convert source image to match the destination image type and format
-	Converted = (ILubyte*)ilConvertBuffer(Src->SizeOfData, Src->Format, Dest->Format, Src->Type, Dest->Type, SrcTemp);
+	Converted = (ILubyte*)ilConvertBuffer(Src->SizeOfData, Src->Format, Dest->Format, Src->Type, Dest->Type, NULL, SrcTemp);
 	if (Converted == NULL)
 		return IL_FALSE;
 	
@@ -695,12 +728,12 @@ ILboolean ILAPIENTRY ilBlit(ILuint Source, ILint DestX,  ILint DestY,   ILint De
 	if (Height + DestY > Dest->Height) Height = Dest->Height - DestY;
 	if (Depth  + DestZ > Dest->Depth)  Depth  = Dest->Depth  - DestZ;
 	
-	// non funziona con rgba
-	if( Src->Format == IL_RGBA || Src->Format == IL_BGRA || Src->Format == IL_LUMINANCE_ALPHA ) {
+	//@TODO: non funziona con rgba
+	if (Src->Format == IL_RGBA || Src->Format == IL_BGRA || Src->Format == IL_LUMINANCE_ALPHA) {
 		const ILuint bpp_without_alpha = Dest->Bpp - 1;
-		for( z = 0; z < Depth; z++ ) {
-			for( y = 0; y < Height; y++ ) {
-				for( x = 0; x < Width; x++ ) {
+		for (z = 0; z < Depth; z++) {
+			for (y = 0; y < Height; y++) {
+				for (x = 0; x < Width; x++) {
 					const ILuint  SrcIndex  = (z+SrcZ)*ConvSizePlane + (y+SrcY)*ConvBps + (x+SrcX)*Dest->Bpp;
 					const ILuint  DestIndex = (z+DestZ)*Dest->SizeOfPlane + (y+DestY)*Dest->Bps + (x+DestX)*Dest->Bpp;
 					const ILuint  AlphaIdx = SrcIndex + bpp_without_alpha;
@@ -804,8 +837,18 @@ ILboolean iCopySubImage(ILimage *Dest, ILimage *Src)
 	return IL_TRUE;
 }
 
+
 ILboolean iCopySubImages(ILimage *Dest, ILimage *Src)
 {
+	if (Src->Faces) {
+		Dest->Faces = (ILimage*)icalloc(1, sizeof(ILimage));
+		if (!Dest->Faces) {
+			return IL_FALSE;
+		}
+		if (!iCopySubImage(Dest->Faces, Src->Faces))
+			return IL_FALSE;
+	}
+	
 	if (Src->Layers) {
 		Dest->Layers = (ILimage*)icalloc(1, sizeof(ILimage));
 		if (!Dest->Layers) {
@@ -814,7 +857,7 @@ ILboolean iCopySubImages(ILimage *Dest, ILimage *Src)
 		if (!iCopySubImage(Dest->Layers, Src->Layers))
 			return IL_FALSE;
 	}
-	
+
 	if (Src->Mipmaps) {
 		Dest->Mipmaps = (ILimage*)icalloc(1, sizeof(ILimage));
 		if (!Dest->Mipmaps) {
@@ -823,7 +866,7 @@ ILboolean iCopySubImages(ILimage *Dest, ILimage *Src)
 		if (!iCopySubImage(Dest->Mipmaps, Src->Mipmaps))
 			return IL_FALSE;
 	}
-	
+
 	if (Src->Next) {
 		Dest->Next = (ILimage*)icalloc(1, sizeof(ILimage));
 		if (!Dest->Next) {
@@ -832,7 +875,7 @@ ILboolean iCopySubImages(ILimage *Dest, ILimage *Src)
 		if (!iCopySubImage(Dest->Next, Src->Next))
 			return IL_FALSE;
 	}
-	
+
 	return IL_TRUE;
 }
 
@@ -849,6 +892,10 @@ ILAPI ILboolean ILAPIENTRY ilCopyImageAttr(ILimage *Dest, ILimage *Src)
 		ifree(Dest->Pal.Palette);
 		Dest->Pal.Palette = NULL;
 	}
+	if (Dest->Faces) {
+		ilCloseImage(Dest->Faces);
+		Dest->Faces = NULL;
+	}
 	if (Dest->Layers) {
 		ilCloseImage(Dest->Layers);
 		Dest->Layers = NULL;
