@@ -1,3 +1,15 @@
+//-----------------------------------------------------------------------------
+//
+// ImageLib Sources
+// Copyright (C) 2000-2009 by Denton Woods
+// Last modified: 03/07/2009
+//
+// Filename: src-IL/src/il_psd.c
+//
+// Description: Reads and writes Photoshop (.psd) files.
+//
+//-----------------------------------------------------------------------------
+
 
 // Information about the .psd format was taken from Adobe's PhotoShop SDK at
 //  http://partners.adobe.com/asn/developer/gapsdk/PhotoshopSDK.html
@@ -189,9 +201,7 @@ ILboolean iLoadPsdInternal()
 		return IL_FALSE;
 	iCurImage->Origin = IL_ORIGIN_UPPER_LEFT;
 
-	ilFixImage();
-
-	return IL_TRUE;
+	return ilFixImage();
 }
 
 
@@ -617,11 +627,23 @@ static ILuint ReadCompressedChannel(const ILuint ChanLen, ILuint Size, ILubyte*
 
 ILboolean PsdGetData(PSDHEAD *Head, void *Buffer, ILboolean Compressed)
 {
-	ILuint		c, x, y, i, Size, ReadResult;
+	ILuint		c, x, y, i, Size, ReadResult, NumChan;
 	ILubyte		*Channel = NULL;
 	ILushort	*ShortPtr;
 	ILuint		*ChanLen = NULL;
 
+	// Added 01-07-2009: This is needed to correctly load greyscale and
+	//  paletted images.
+	switch (Head->Mode)
+	{
+		case 1:
+		case 2:
+			NumChan = 1;
+			break;
+		default:
+			NumChan = 3;
+	}
+
 	Channel = (ILubyte*)ialloc(Head->Width * Head->Height * iCurImage->Bpc);
 	if (Channel == NULL) {
 		return IL_FALSE;
@@ -637,7 +659,7 @@ ILboolean PsdGetData(PSDHEAD *Head, void *Buffer, ILboolean Compressed)
 
 	if (!Compressed) {
 		if (iCurImage->Bpc == 1) {
-			for (c = 0; c < 3; c++) {
+			for (c = 0; c < NumChan; c++) {
 				i = 0;
 				if (iread(Channel, Head->Width * Head->Height, 1) != 1) {
 					ifree(Channel);
@@ -650,6 +672,7 @@ ILboolean PsdGetData(PSDHEAD *Head, void *Buffer, ILboolean Compressed)
 				}
 			}
 			// Accumulate any remaining channels into a single alpha channel
+			//@TODO: This needs to be changed for greyscale images.
 			for (; c < Head->Channels; c++) {
 				i = 0;
 				if (iread(Channel, Head->Width * Head->Height, 1) != 1) {
@@ -666,7 +689,7 @@ ILboolean PsdGetData(PSDHEAD *Head, void *Buffer, ILboolean Compressed)
 			}
 		}
 		else {  // iCurImage->Bpc == 2
-			for (c = 0; c < 3; c++) {
+			for (c = 0; c < NumChan; c++) {
 				i = 0;
 				if (iread(Channel, Head->Width * Head->Height * 2, 1) != 1) {
 					ifree(Channel);
@@ -684,6 +707,7 @@ ILboolean PsdGetData(PSDHEAD *Head, void *Buffer, ILboolean Compressed)
 				iCurImage->Bps *= 2;
 			}
 			// Accumulate any remaining channels into a single alpha channel
+			//@TODO: This needs to be changed for greyscale images.
 			for (; c < Head->Channels; c++) {
 				i = 0;
 				if (iread(Channel, Head->Width * Head->Height * 2, 1) != 1) {
@@ -706,7 +730,7 @@ ILboolean PsdGetData(PSDHEAD *Head, void *Buffer, ILboolean Compressed)
 		ChanLen = GetCompChanLen(Head);
 
 		Size = Head->Width * Head->Height;
-		for (c = 0; c < 3; c++) {
+		for (c = 0; c < NumChan; c++) {
 			ReadResult = ReadCompressedChannel(ChanLen[c], Size, Channel);
 			if (ReadResult == READ_COMPRESSED_ERROR_FILE_CORRUPT)
 				goto file_corrupt;
@@ -722,6 +746,7 @@ ILboolean PsdGetData(PSDHEAD *Head, void *Buffer, ILboolean Compressed)
 		}
 
 		// Initialize the alpha channel to solid
+		//@TODO: This needs to be changed for greyscale images.
 		if (Head->Channels >= 4) {
 			for (y = 0; y < Head->Height * iCurImage->Bps; y += iCurImage->Bps) {
 				for (x = 0; x < iCurImage->Bps; x += iCurImage->Bpp) {
@@ -778,7 +803,7 @@ ILboolean ParseResources(ILuint ResourceSize, ILubyte *Resources)
 		return IL_FALSE;
 	}
 
-	while (ResourceSize) {
+	while (ResourceSize > 13) {  // Absolutely has to be larger than this.
 		if (strncmp("8BIM", (const char*)Resources, 4)) {
 			//return IL_FALSE;
 			return IL_TRUE;  // 05-30-2002: May not necessarily mean corrupt data...
@@ -799,11 +824,15 @@ ILboolean ParseResources(ILuint ResourceSize, ILubyte *Resources)
 		BigUInt(&Size);
 		Resources += 4;
 
-		ResourceSize -= (4 + 2 + 1 + NameLen);
+		ResourceSize -= (4 + 2 + 1 + NameLen + 4);
 
 		switch (ID)
 		{
 			case 0x040F:  // ICC Profile
+				if (Size > ResourceSize) {  // Check to make sure we are not going past the end of Resources.
+					ilSetError(IL_ILLEGAL_FILE_VALUE);
+					return IL_FALSE;
+				}
 				iCurImage->Profile = (ILubyte*)ialloc(Size);
 				if (iCurImage->Profile == NULL) {
 					return IL_FALSE;
@@ -872,10 +901,10 @@ ILboolean GetSingleChannel(PSDHEAD *Head, ILubyte *Buffer, ILboolean Compressed)
 
 
 //! Writes a Psd file
-ILboolean ilSavePsd(ILconst_string FileName)
+ILboolean ilSavePsd(const ILstring FileName)
 {
 	ILHANDLE	PsdFile;
-	ILboolean	bPsd = IL_FALSE;
+	ILuint		PsdSize;
 
 	if (ilGetBoolean(IL_FILE_MODE) == IL_FALSE) {
 		if (iFileExists(FileName)) {
@@ -887,29 +916,39 @@ ILboolean ilSavePsd(ILconst_string FileName)
 	PsdFile = iopenw(FileName);
 	if (PsdFile == NULL) {
 		ilSetError(IL_COULD_NOT_OPEN_FILE);
-		return bPsd;
+		return IL_FALSE;
 	}
 
-	bPsd = ilSavePsdF(PsdFile);
+	PsdSize = ilSavePsdF(PsdFile);
 	iclosew(PsdFile);
 
-	return bPsd;
+	if (PsdSize == 0)
+		return IL_FALSE;
+	return IL_TRUE;
 }
 
 
 //! Writes a Psd to an already-opened file
-ILboolean ilSavePsdF(ILHANDLE File)
+ILuint ilSavePsdF(ILHANDLE File)
 {
+	ILuint Pos;
 	iSetOutputFile(File);
-	return iSavePsdInternal();
+	Pos = itellw();
+	if (iSavePsdInternal() == IL_FALSE)
+		return 0;  // Error occurred
+	return itellw() - Pos;  // Return the number of bytes written.
 }
 
 
 //! Writes a Psd to a memory "lump"
-ILboolean ilSavePsdL(void *Lump, ILuint Size)
+ILuint ilSavePsdL(void *Lump, ILuint Size)
 {
+	ILuint Pos;
 	iSetOutputLump(Lump, Size);
-	return iSavePsdInternal();
+	Pos = itellw();
+	if (iSavePsdInternal() == IL_FALSE)
+		return 0;  // Error occurred
+	return itellw() - Pos;  // Return the number of bytes written.
 }
 
 
@@ -1000,7 +1039,7 @@ ILboolean iSavePsdInternal()
 
 	SaveBigInt(0);  // No image resources.
 	SaveBigInt(0);  // No layer information.
-	SaveBigShort(0);  // Raw data, no compression.
+	SaveBigShort(0);  // Psd data, no compression.
 
 	// @TODO:  Add RLE compression.
 
