@@ -1,8 +1,8 @@
 //-----------------------------------------------------------------------------
 //
 // ImageLib Sources
-// Copyright (C) 2000-2008 by Denton Woods
-// Last modified: 12/17/2008
+// Copyright (C) 2000-2009 by Denton Woods
+// Last modified: 03/07/2009
 //
 // Filename: src-IL/src/il_states.c
 //
@@ -19,20 +19,25 @@
 //#include <malloc.h>
 #include <stdlib.h>
 
-ILstring _ilVendor		= IL_TEXT("Abysmal Software");
-ILstring _ilVersion		= IL_TEXT("Developer's Image Library (DevIL) 1.7.5");
+ILconst_string _ilVendor		= IL_TEXT("Abysmal Software");
+ILconst_string _ilVersion		= IL_TEXT("Developer's Image Library (DevIL) 1.7.8");
 
 
-char* _ilLoadExt		= "" IL_BMP_EXT IL_CUT_EXT IL_DCX_EXT IL_DDS_EXT
-									IL_GIF_EXT IL_HDR_EXT IL_ICO_EXT IL_JPG_EXT IL_LIF_EXT
-									IL_MDL_EXT IL_MNG_EXT IL_PCX_EXT IL_PIC_EXT
-									IL_PIX_EXT IL_PNG_EXT IL_PNM_EXT IL_PSD_EXT
-									IL_PSP_EXT IL_PXR_EXT IL_SGI_EXT IL_TGA_EXT
-									IL_TIF_EXT IL_WAL_EXT IL_XPM_EXT IL_JP2_EXT
-									IL_ICNS_EXT IL_VTF_EXT;
-char* _ilSaveExt		= "" IL_BMP_EXT IL_CHEAD_EXT IL_DDS_EXT IL_JPG_EXT
-									IL_PCX_EXT IL_PNG_EXT IL_PNM_EXT IL_PSD_EXT
-									IL_SGI_EXT IL_TGA_EXT IL_TIF_EXT;
+char* _ilLoadExt		= "" IL_BLP_EXT IL_BMP_EXT IL_CUT_EXT IL_DCX_EXT IL_DDS_EXT
+									IL_DCM_EXT IL_DPX_EXT IL_EXR_EXT IL_FITS_EXT IL_FTX_EXT
+									IL_GIF_EXT IL_HDR_EXT IL_ICNS_EXT IL_ICO_EXT IL_IFF_EXT
+									IL_IWI_EXT IL_JPG_EXT IL_JP2_EXT IL_LIF_EXT IL_MDL_EXT
+									IL_MNG_EXT IL_MP3_EXT IL_PCD_EXT IL_PCX_EXT IL_PIC_EXT
+									IL_PIX_EXT IL_PNG_EXT IL_PNM_EXT IL_PSD_EXT IL_PSP_EXT
+									IL_PXR_EXT IL_RAW_EXT IL_ROT_EXT IL_SGI_EXT IL_SUN_EXT
+									IL_TEX_EXT IL_TGA_EXT IL_TIF_EXT IL_TPL_EXT IL_UTX_EXT
+									IL_VTF_EXT IL_WAL_EXT IL_WDP_EXT IL_XPM_EXT;
+
+char* _ilSaveExt		= "" IL_BMP_EXT IL_CHEAD_EXT IL_DDS_EXT IL_EXR_EXT
+									IL_HDR_EXT IL_JP2_EXT IL_JPG_EXT IL_PCX_EXT
+									IL_PNG_EXT IL_PNM_EXT IL_PSD_EXT IL_RAW_EXT
+									IL_SGI_EXT IL_TGA_EXT IL_TIF_EXT IL_VTF_EXT
+									IL_WBMP_EXT;
 
 
 //! Set all states to their defaults.
@@ -63,6 +68,7 @@ void ilDefaultStates()
 	ilStates[ilCurrentPos].ilDxtcFormat = IL_DXT1;
 	ilStates[ilCurrentPos].ilPcdPicNum = 2;
 	ilStates[ilCurrentPos].ilPngAlphaIndex = -1;
+	ilStates[ilCurrentPos].ilVtfCompression = IL_DXT_NO_COMP;
 
 	ilStates[ilCurrentPos].ilTgaId = NULL;
 	ilStates[ilCurrentPos].ilTgaAuthName = NULL;
@@ -83,6 +89,8 @@ void ilDefaultStates()
 	ilStates[ilCurrentPos].ilQuantMaxIndexs = 256;
 
 	ilStates[ilCurrentPos].ilKeepDxtcData = IL_FALSE;
+	ilStates[ilCurrentPos].ilUseNVidiaDXT = IL_FALSE;
+	ilStates[ilCurrentPos].ilUseSquishDXT = IL_FALSE;
 
 
 
@@ -97,41 +105,41 @@ void ilDefaultStates()
 
 
 //! Returns a constant string detailing aspects about this library.
-ILstring ILAPIENTRY ilGetString(ILenum StringName)
+ILconst_string ILAPIENTRY ilGetString(ILenum StringName)
 {
 	switch (StringName)
 	{
 		case IL_VENDOR:
-			return (ILstring)_ilVendor;
+			return (ILconst_string)_ilVendor;
 		case IL_VERSION_NUM: //changed 2003-08-30: IL_VERSION changes									//switch define ;-)
-			return (ILstring)_ilVersion;
+			return (ILconst_string)_ilVersion;
 		case IL_LOAD_EXT:
-			return (ILstring)_ilLoadExt;
+			return (ILconst_string)_ilLoadExt;
 		case IL_SAVE_EXT:
-			return (ILstring)_ilSaveExt;
+			return (ILconst_string)_ilSaveExt;
 		case IL_TGA_ID_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilTgaId;
+			return (ILconst_string)ilStates[ilCurrentPos].ilTgaId;
 		case IL_TGA_AUTHNAME_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilTgaAuthName;
+			return (ILconst_string)ilStates[ilCurrentPos].ilTgaAuthName;
 		case IL_TGA_AUTHCOMMENT_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilTgaAuthComment;
+			return (ILconst_string)ilStates[ilCurrentPos].ilTgaAuthComment;
 		case IL_PNG_AUTHNAME_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilPngAuthName;
+			return (ILconst_string)ilStates[ilCurrentPos].ilPngAuthName;
 		case IL_PNG_TITLE_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilPngTitle;
+			return (ILconst_string)ilStates[ilCurrentPos].ilPngTitle;
 		case IL_PNG_DESCRIPTION_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilPngDescription;
+			return (ILconst_string)ilStates[ilCurrentPos].ilPngDescription;
 		//2003-08-31: added tif strings
 		case IL_TIF_DESCRIPTION_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilTifDescription;
+			return (ILconst_string)ilStates[ilCurrentPos].ilTifDescription;
 		case IL_TIF_HOSTCOMPUTER_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilTifHostComputer;
+			return (ILconst_string)ilStates[ilCurrentPos].ilTifHostComputer;
 		case IL_TIF_DOCUMENTNAME_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilTifDocumentName;
+			return (ILconst_string)ilStates[ilCurrentPos].ilTifDocumentName;
 		case IL_TIF_AUTHNAME_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilTifAuthName;
+			return (ILconst_string)ilStates[ilCurrentPos].ilTifAuthName;
 		case IL_CHEAD_HEADER_STRING:
-			return (ILstring)ilStates[ilCurrentPos].ilCHeader;
+			return (ILconst_string)ilStates[ilCurrentPos].ilCHeader;
 		default:
 			ilSetError(IL_INVALID_ENUM);
 			break;
@@ -247,6 +255,13 @@ ILboolean ilAble(ILenum Mode, ILboolean Flag)
 			break;
 		case IL_JPG_PROGRESSIVE:
 			ilStates[ilCurrentPos].ilJpgProgressive = Flag;
+			break;
+		case IL_NVIDIA_COMPRESS:
+			ilStates[ilCurrentPos].ilUseNVidiaDXT = Flag;
+			break;
+		case IL_SQUISH_COMPRESS:
+			ilStates[ilCurrentPos].ilUseSquishDXT = Flag;
+			break;
 
 		default:
 			ilSetError(IL_INVALID_ENUM);
@@ -282,6 +297,10 @@ ILboolean ILAPIENTRY ilIsEnabled(ILenum Mode)
 			return ilStates[ilCurrentPos].ilInterlace;
 		case IL_JPG_PROGRESSIVE:
 			return ilStates[ilCurrentPos].ilJpgProgressive;
+		case IL_NVIDIA_COMPRESS:
+			return ilStates[ilCurrentPos].ilUseNVidiaDXT;
+		case IL_SQUISH_COMPRESS:
+			return ilStates[ilCurrentPos].ilUseSquishDXT;
 
 		default:
 			ilSetError(IL_INVALID_ENUM);
@@ -324,6 +343,8 @@ ILboolean ILAPIENTRY ilGetBoolean(ILenum Mode)
 
 ILimage *iGetBaseImage(void);
 
+//! Internal function to figure out where we are in an image chain.
+//@TODO: This may get much more complex with mipmaps under faces, etc.
 ILuint iGetActiveNum(ILenum Type)
 {
 	ILimage *BaseImage;
@@ -335,7 +356,6 @@ ILuint iGetActiveNum(ILenum Type)
 	}
 
 	BaseImage = iGetBaseImage();
-
 	if (BaseImage == iCurImage)
 		return 0;
 
@@ -343,31 +363,55 @@ ILuint iGetActiveNum(ILenum Type)
 	{
 		case IL_ACTIVE_IMAGE:
 			BaseImage = BaseImage->Next;
+			do {
+				if (BaseImage == NULL)
+					return 0;
+				Num++;
+				if (BaseImage == iCurImage)
+					return Num;
+			} while ((BaseImage = BaseImage->Next));
 			break;
 		case IL_ACTIVE_MIPMAP:
 			BaseImage = BaseImage->Mipmaps;
+			do {
+				if (BaseImage == NULL)
+					return 0;
+				Num++;
+				if (BaseImage == iCurImage)
+					return Num;
+			} while ((BaseImage = BaseImage->Mipmaps));
 			break;
 		case IL_ACTIVE_LAYER:
 			BaseImage = BaseImage->Layers;
+			do {
+				if (BaseImage == NULL)
+					return 0;
+				Num++;
+				if (BaseImage == iCurImage)
+					return Num;
+			} while ((BaseImage = BaseImage->Layers));
+			break;
+		case IL_ACTIVE_FACE:
+			BaseImage = BaseImage->Faces;
+			do {
+				if (BaseImage == NULL)
+					return 0;
+				Num++;
+				if (BaseImage == iCurImage)
+					return Num;
+			} while ((BaseImage = BaseImage->Faces));
 			break;
 	}
 
-	do {
-		if (BaseImage == NULL)
-			return 0;
-		Num++;
-		if (BaseImage == iCurImage)
-			return Num;
-	} while ((BaseImage = BaseImage->Next));
-
-	//ilSetError(IL_ILLEGAL_OPERATION);
+	//@TODO: Any error needed here?
 
 	return 0;
 }
 
 
 //! Sets Param equal to the current value of the Mode
-void ILAPIENTRY ilGetIntegerv(ILenum Mode, ILint *Param) {
+void ILAPIENTRY ilGetIntegerv(ILenum Mode, ILint *Param)
+{
 	if (Param == NULL) {
 		ilSetError(IL_INVALID_PARAM);
 		return;
@@ -399,7 +443,7 @@ void ILAPIENTRY ilGetIntegerv(ILenum Mode, ILint *Param) {
 		case IL_ORIGIN_MODE:
 			*Param = ilStates[ilCurrentPos].ilOriginMode;
 			break;
-		case IL_MAX_QUANT_INDEXS:
+		case IL_MAX_QUANT_INDICES:
 			*Param = ilStates[ilCurrentPos].ilQuantMaxIndexs;
 			break;
 		case IL_NEU_QUANT_SAMPLE:
@@ -453,6 +497,9 @@ void ILAPIENTRY ilGetIntegerv(ILenum Mode, ILint *Param) {
 		case IL_TGA_RLE:
 			*Param = ilStates[ilCurrentPos].ilTgaRle;
 			break;
+		case IL_VTF_COMP:
+			*Param = ilStates[ilCurrentPos].ilVtfCompression;
+			break;
 
 		// Boolean values
 		case IL_CONV_PAL:
@@ -482,6 +529,12 @@ void ILAPIENTRY ilGetIntegerv(ILenum Mode, ILint *Param) {
 		case IL_JPG_PROGRESSIVE:
 			*Param = ilStates[ilCurrentPos].ilJpgProgressive;
 			break;
+		case IL_NVIDIA_COMPRESS:
+			*Param = ilStates[ilCurrentPos].ilUseNVidiaDXT;
+			break;
+		case IL_SQUISH_COMPRESS:
+			*Param = ilStates[ilCurrentPos].ilUseSquishDXT;
+			break;
 
 		default:
             iGetIntegervImage(iCurImage, Mode, Param);
@@ -492,7 +545,8 @@ void ILAPIENTRY ilGetIntegerv(ILenum Mode, ILint *Param) {
 
 //@TODO rename to ilGetImageIntegerv for 1.6.9 and make it public
 //! Sets Param equal to the current value of the Mode
-void ILAPIENTRY iGetIntegervImage(ILimage *Image, ILenum Mode, ILint *Param) {
+void ILAPIENTRY iGetIntegervImage(ILimage *Image, ILenum Mode, ILint *Param)
+{
     ILimage *SubImage;
     if (Image == NULL) {
         ilSetError(IL_ILLEGAL_OPERATION);
@@ -503,6 +557,7 @@ void ILAPIENTRY iGetIntegervImage(ILimage *Image, ILenum Mode, ILint *Param) {
         return;
     }
     *Param = 0;
+
     switch (Mode)
     {
         case IL_DXTC_DATA_FORMAT:
@@ -514,11 +569,11 @@ void ILAPIENTRY iGetIntegervImage(ILimage *Image, ILenum Mode, ILint *Param) {
             break;
             ////
         case IL_IMAGE_BITS_PER_PIXEL:
-            //changed 20040610 to channel count (Bpp) times Byte per channel
+            //changed 20040610 to channel count (Bpp) times Bytes per channel
             *Param = (Image->Bpp << 3)*Image->Bpc;
             break;
         case IL_IMAGE_BYTES_PER_PIXEL:
-            //changed 20040610 to channel count (Bpp) times Byte per channel
+            //changed 20040610 to channel count (Bpp) times Bytes per channel
             *Param = Image->Bpp*Image->Bpc;
             break;
         case IL_IMAGE_BPC:
@@ -564,16 +619,20 @@ void ILAPIENTRY iGetIntegervImage(ILimage *Image, ILenum Mode, ILint *Param) {
         case IL_IMAGE_WIDTH:
             *Param = Image->Width;
             break;
+        case IL_NUM_FACES:
+            for (SubImage = Image->Faces; SubImage; SubImage = SubImage->Faces)
+                (*Param)++;
+            break;
         case IL_NUM_IMAGES:
-            for(SubImage = Image->Next; SubImage; SubImage = SubImage->Next)
+            for (SubImage = Image->Next; SubImage; SubImage = SubImage->Next)
                 (*Param)++;
             break;
         case IL_NUM_LAYERS:
-            for(SubImage = Image->Layers; SubImage; SubImage = SubImage->Next)
+            for (SubImage = Image->Layers; SubImage; SubImage = SubImage->Layers)
                 (*Param)++;
             break;
         case IL_NUM_MIPMAPS:
-            for(SubImage = Image->Mipmaps; SubImage; SubImage = SubImage->Next)
+			for (SubImage = Image->Mipmaps; SubImage; SubImage = SubImage->Mipmaps)
                 (*Param)++;
             break;
 
@@ -692,7 +751,7 @@ ILboolean ILAPIENTRY ilCompressFunc(ILenum Mode)
 		case IL_COMPRESS_NONE:
 		case IL_COMPRESS_RLE:
 		//case IL_COMPRESS_LZO:
-			case IL_COMPRESS_ZLIB:
+		case IL_COMPRESS_ZLIB:
 			ilStates[ilCurrentPos].ilCompression = Mode;
 			break;
 		default:
@@ -978,7 +1037,7 @@ void ILAPIENTRY ilSetInteger(ILenum Mode, ILint Param)
 				return;
 			}
 			break;
-		case IL_MAX_QUANT_INDEXS:
+		case IL_MAX_QUANT_INDICES:
 			if (Param >= 2 && Param <= 256) {
 				ilStates[ilCurrentPos].ilQuantMaxIndexs = Param;
 				return;
@@ -1041,7 +1100,7 @@ void ILAPIENTRY ilSetInteger(ILenum Mode, ILint Param)
 			}
 			break;
 		case IL_DXTC_FORMAT:
-			if (Param >= IL_DXT1 || Param <= IL_DXT5) {
+			if (Param >= IL_DXT1 || Param <= IL_DXT5 || Param == IL_DXT1A) {
 				ilStates[ilCurrentPos].ilDxtcFormat = Param;
 				return;
 			}
@@ -1094,6 +1153,12 @@ void ILAPIENTRY ilSetInteger(ILenum Mode, ILint Param)
 				return;
 			}
 			break;
+		case IL_VTF_COMP:
+			if (Param == IL_DXT1 || Param == IL_DXT5 || Param == IL_DXT3 || Param == IL_DXT1A || Param == IL_DXT_NO_COMP) {
+				ilStates[ilCurrentPos].ilVtfCompression = Param;
+				return;
+			}
+			break;
 
 		default:
 			ilSetError(IL_INVALID_ENUM);
