@@ -1,12 +1,12 @@
 //-----------------------------------------------------------------------------
 //
 // ImageLib Sources
-// Copyright (C) 2000-2008 by Denton Woods
-// Last modified: 12/14/2008
+// Copyright (C) 2000-2009 by Denton Woods
+// Last modified: 03/07/2009
 //
 // Filename: src-IL/src/il_targa.c
 //
-// Description: Reads from and writes to a targa (.tga) file.
+// Description: Reads from and writes to a Targa (.tga) file.
 //
 //-----------------------------------------------------------------------------
 
@@ -120,7 +120,7 @@ ILboolean iCheckTarga(TARGAHEAD *Header)
 		return IL_FALSE;
 	
 	// check type (added 20040218)
-	if(Header->ImageType != TGA_NO_DATA
+	if (Header->ImageType   != TGA_NO_DATA
 	   && Header->ImageType != TGA_COLMAP_UNCOMP
 	   && Header->ImageType != TGA_UNMAP_UNCOMP
 	   && Header->ImageType != TGA_BW_UNCOMP
@@ -157,7 +157,8 @@ ILboolean ilLoadTarga(ILconst_string FileName)
 
 
 //! Reads an already-opened Targa file
-ILboolean ilLoadTargaF(ILHANDLE File) {
+ILboolean ilLoadTargaF(ILHANDLE File)
+{
 	ILuint		FirstPos;
 	ILboolean	bRet;
 	
@@ -244,9 +245,7 @@ ILboolean iLoadTargaInternal()
 			break;
 	}
 	
-	ilFixImage();
-	
-	return bTarga;
+	return ilFixImage();
 }
 
 
@@ -441,7 +440,7 @@ ILboolean iReadBwTga(TARGAHEAD *Header)
 
 ILboolean iUncompressTgaData(ILimage *Image)
 {
-	ILuint	BytesRead = 0, Size, RunLen, i;
+	ILuint	BytesRead = 0, Size, RunLen, i, ToRead;
 	ILubyte Header, Color[4];
 	ILint	c;
 	
@@ -460,7 +459,8 @@ ILboolean iUncompressTgaData(ILimage *Image)
 			}
 			RunLen = (Header+1) * Image->Bpp;
 			for (i = 0; i < RunLen; i += Image->Bpp) {
-				for (c = 0; c < Image->Bpp; c++) {
+				// Read the color in, but we check to make sure that we do not go past the end of the image.
+				for (c = 0; c < Image->Bpp && BytesRead+i+c < Size; c++) {
 					Image->Data[BytesRead+i+c] = Color[c];
 				}
 			}
@@ -468,11 +468,19 @@ ILboolean iUncompressTgaData(ILimage *Image)
 		}
 		else {
 			RunLen = (Header+1) * Image->Bpp;
-			if (iread(Image->Data + BytesRead, 1, RunLen) != RunLen) {
-				iUnCache();
+			// We have to check that we do not go past the end of the image data.
+			if (BytesRead + RunLen > Size)
+				ToRead = Size - BytesRead;
+			else
+				ToRead = RunLen;
+			if (iread(Image->Data + BytesRead, 1, ToRead) != ToRead) {
+				iUnCache();  //@TODO: Error needed here?
 				return IL_FALSE;
 			}
 			BytesRead += RunLen;
+
+			if (BytesRead + RunLen > Size)
+				iseek(RunLen - ToRead, IL_SEEK_CUR);
 		}
 	}
 	
@@ -531,53 +539,62 @@ ILboolean i16BitTarga(ILimage *Image)
 
 
 //! Writes a Targa file
-ILboolean ilSaveTarga(ILconst_string FileName)
+ILboolean ilSaveTarga(const ILstring FileName)
 {
 	ILHANDLE	TargaFile;
-	ILboolean	bTarga = IL_FALSE;
-	
+	ILuint		TargaSize;
+
 	if (ilGetBoolean(IL_FILE_MODE) == IL_FALSE) {
 		if (iFileExists(FileName)) {
 			ilSetError(IL_FILE_ALREADY_EXISTS);
 			return IL_FALSE;
 		}
 	}
-	
+
 	TargaFile = iopenw(FileName);
 	if (TargaFile == NULL) {
 		ilSetError(IL_COULD_NOT_OPEN_FILE);
-		return bTarga;
+		return IL_FALSE;
 	}
-	
-	bTarga = ilSaveTargaF(TargaFile);
+
+	TargaSize = ilSaveTargaF(TargaFile);
 	iclosew(TargaFile);
-	
-	return bTarga;
+
+	if (TargaSize == 0)
+		return IL_FALSE;
+	return IL_TRUE;
 }
 
 
 //! Writes a Targa to an already-opened file
-ILboolean ilSaveTargaF(ILHANDLE File)
+ILuint ilSaveTargaF(ILHANDLE File)
 {
+	ILuint Pos;
 	iSetOutputFile(File);
-	return iSaveTargaInternal();
+	Pos = itellw();
+	if (iSaveTargaInternal() == IL_FALSE)
+		return 0;  // Error occurred
+	return itellw() - Pos;  // Return the number of bytes written.
 }
 
 
 //! Writes a Targa to a memory "lump"
-ILboolean ilSaveTargaL(void *Lump, ILuint Size)
+ILuint ilSaveTargaL(void *Lump, ILuint Size)
 {
+	ILuint Pos = itellw();
 	iSetOutputLump(Lump, Size);
-	return iSaveTargaInternal();
+	if (iSaveTargaInternal() == IL_FALSE)
+		return 0;  // Error occurred
+	return itellw() - Pos;  // Return the number of bytes written.
 }
 
 
 // Internal function used to save the Targa.
 ILboolean iSaveTargaInternal()
 {
-	const char		*ID = iGetString(IL_TGA_ID_STRING);
-	const char		*AuthName = iGetString(IL_TGA_AUTHNAME_STRING);
-	const char		*AuthComment = iGetString(IL_TGA_AUTHCOMMENT_STRING);
+	const char	*ID = iGetString(IL_TGA_ID_STRING);
+	const char	*AuthName = iGetString(IL_TGA_AUTHNAME_STRING);
+	const char	*AuthComment = iGetString(IL_TGA_AUTHCOMMENT_STRING);
 	ILubyte 	IDLen = 0, UsePal, Type, PalEntSize;
 	ILshort 	ColMapStart = 0, PalSize;
 	ILubyte		Temp;
@@ -592,7 +609,7 @@ ILboolean iSaveTargaInternal()
 	char		*idString = "Developer's Image Library (DevIL)";
 	ILuint		Day, Month, Year, Hour, Minute, Second;
 	char		*TempData;
-	
+
 	if (iCurImage == NULL) {
 		ilSetError(IL_ILLEGAL_OPERATION);
 		return IL_FALSE;
@@ -613,7 +630,7 @@ ILboolean iSaveTargaInternal()
 	
 	iwrite(&IDLen, sizeof(ILubyte), 1);
 	iwrite(&UsePal, sizeof(ILubyte), 1);
-	
+
 	Format = iCurImage->Format;
 	switch (Format) {
 		case IL_COLOUR_INDEX:
@@ -815,6 +832,52 @@ ILboolean iSaveTargaInternal()
 }
 
 
+// Only to be called by ilDetermineSize.  Returns the buffer size needed to save the
+//  current image as a Targa file.
+ILuint iTargaSize(void)
+{
+	ILuint	Size, Bpp;
+	ILubyte	IDLen = 0;
+	const char	*ID = iGetString(IL_TGA_ID_STRING);
+	const char	*AuthName = iGetString(IL_TGA_AUTHNAME_STRING);
+	const char	*AuthComment = iGetString(IL_TGA_AUTHCOMMENT_STRING);
+
+	//@TODO: Support color indexed images.
+	if (iGetInt(IL_TGA_RLE) == IL_TRUE || iCurImage->Format == IL_COLOUR_INDEX) {
+		// Use the slower method, since we are using compression.  We do a "fake" write.
+		ilSaveTargaL(NULL, 0);
+	}
+
+	if (ID)
+		IDLen = (ILubyte)ilCharStrLen(ID);
+
+	Size = 18 + IDLen;  // Header + ID
+
+	// Bpp may not be iCurImage->Bpp.
+	switch (iCurImage->Format)
+	{
+		case IL_BGR:
+		case IL_RGB:
+			Bpp = 3;
+			break;
+		case IL_BGRA:
+		case IL_RGBA:
+			Bpp = 4;
+			break;
+		case IL_LUMINANCE:
+			Bpp = 1;
+			break;
+		default:  //@TODO: Do not know what to do with the others yet.
+			return 0;
+	}
+
+	Size += iCurImage->Width * iCurImage->Height * Bpp;
+	Size += 532;  // Size of the extension area
+
+	return Size;
+}
+
+
 /*// Makes a neat string to go into the id field of the .tga
 void iMakeString(char *Str)
 {
