@@ -1,8 +1,8 @@
 //-----------------------------------------------------------------------------
 //
 // ImageLib Sources
-// Copyright (C) 2000-2008 by Denton Woods
-// Last modified: 12/14/2008
+// Copyright (C) 2000-2009 by Denton Woods
+// Last modified: 03/07/2009
 //
 // Filename: src-IL/src/il_tiff.c
 //
@@ -40,9 +40,7 @@
 static ILboolean iLoadTiffInternal(void);
 static char*     iMakeString(void);
 static TIFF*     iTIFFOpen(char *Mode);
-//static ILboolean iSaveTiffInternal();
-
-static ILboolean iSaveTiffInternal(ILconst_string Filename);
+static ILboolean iSaveTiffInternal(/*ILconst_string Filename*/);
 
 /*----------------------------------------------------------------------------*/
 
@@ -364,7 +362,7 @@ ILboolean iLoadTiffInternal()
 			if (!Image) {
 				int type = IL_UNSIGNED_BYTE;
 				if (bitspersample == 16) type = IL_UNSIGNED_SHORT;
-				if(!ilTexImage(w, h, 1, 1, IL_LUMINANCE, type, NULL)) {
+				if (!ilTexImage(w, h, 1, 1, IL_LUMINANCE, type, NULL)) {
 					TIFFClose(tif);
 					return IL_FALSE;
 				}
@@ -372,7 +370,7 @@ ILboolean iLoadTiffInternal()
 			}
 			else {
 				Image->Next = ilNewImage(w, h, 1, 1, 1);
-				if(Image->Next == NULL) {
+				if (Image->Next == NULL) {
 					TIFFClose(tif);
 					return IL_FALSE;
 				}
@@ -390,7 +388,7 @@ ILboolean iLoadTiffInternal()
 				Image->Format = IL_COLOUR_INDEX;
 				Image->Pal.PalSize = (count)*3;
 				Image->Pal.PalType = IL_PAL_RGB24;
-				Image->Pal.Palette = ialloc(Image->Pal.PalSize);
+				Image->Pal.Palette = (ILubyte*)ialloc(Image->Pal.PalSize);
 				entry = Image->Pal.Palette;
 				for (j = 0; j < count; ++j) {
 					entry[0] = (ILubyte)(red[j] >> 8);
@@ -404,7 +402,7 @@ ILboolean iLoadTiffInternal()
 			TIFFGetField(tif, TIFFTAG_ROWSPERSTRIP, &rowsperstrip);
 			stripsize = TIFFStripSize(tif);
 
-			strip = ialloc(stripsize);
+			strip = (ILubyte*)ialloc(stripsize);
 
 			if (bitspersample == 8 || bitspersample == 16) {
 				ILubyte *dat = Image->Data;
@@ -527,7 +525,7 @@ ILboolean iLoadTiffInternal()
 			TIFFGetField(tif, TIFFTAG_ROWSPERSTRIP, &rowsperstrip);
 			stripsize = TIFFStripSize(tif);
 
-			strip = ialloc(stripsize);
+			strip = (ILubyte*)ialloc(stripsize);
 
 			dat = Image->Data;
 			for (y = 0; y < h; y += rowsperstrip) {
@@ -715,9 +713,7 @@ ILboolean iLoadTiffInternal()
 
 	TIFFClose(tif);
 
-	ilFixImage();
-
-	return IL_TRUE;
+	return ilFixImage();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -736,13 +732,23 @@ _tiffFileReadProc(thandle_t fd, tdata_t pData, tsize_t tSize)
 
 /*----------------------------------------------------------------------------*/
 
+// We have trouble sometimes reading when writing a file.  Specifically, the only time
+//  I have seen libtiff call this is at the beginning of writing a tiff, before
+//  anything is ever even written!  Also, we have to return 0, no matter what tSize
+//  is.  If we return tSize like would be expected, then TIFFClientOpen fails.
+static tsize_t 
+_tiffFileReadProcW(thandle_t fd, tdata_t pData, tsize_t tSize)
+{
+	fd;
+	return 0;
+}
+
+/*----------------------------------------------------------------------------*/
+
 static tsize_t 
 _tiffFileWriteProc(thandle_t fd, tdata_t pData, tsize_t tSize)
 {
 	fd;
-	/*TIFFWarning("TIFFMemFile", "_tiffFileWriteProc() Not implemented");
-	return(0);
-	*/
 	return iwrite(pData, 1, tSize);
 }
 
@@ -757,12 +763,12 @@ _tiffFileSeekProc(thandle_t fd, toff_t tOff, int whence)
 		return 0xFFFFFFFF;
 
 	iseek(tOff, whence);
-	return tOff;
+	return itell();
+	//return tOff;
 }
 
 /*----------------------------------------------------------------------------*/
 
-#if 0
 static toff_t
 _tiffFileSeekProcW(thandle_t fd, toff_t tOff, int whence)
 {
@@ -771,9 +777,10 @@ _tiffFileSeekProcW(thandle_t fd, toff_t tOff, int whence)
 		return 0xFFFFFFFF;
 
 	iseekw(tOff, whence);
-	return tOff;
+	return itellw();
+	//return tOff;
 }
-#endif
+
 /*----------------------------------------------------------------------------*/
 
 static int
@@ -801,7 +808,6 @@ _tiffFileSizeProc(thandle_t fd)
 
 /*----------------------------------------------------------------------------*/
 
-#if 0
 static toff_t
 _tiffFileSizeProcW(thandle_t fd)
 {
@@ -813,7 +819,7 @@ _tiffFileSizeProcW(thandle_t fd)
 
 	return Size;
 }
-#endif
+
 /*----------------------------------------------------------------------------*/
 
 #ifdef __BORLANDC__
@@ -843,16 +849,15 @@ _tiffDummyUnmapProc(thandle_t fd, tdata_t base, toff_t size)
 TIFF *iTIFFOpen(char *Mode)
 {
 	TIFF *tif;
-#if 0
+
 	if (Mode[0] == 'w')
 		tif = TIFFClientOpen("TIFFMemFile", Mode,
 							NULL,
-							_tiffFileReadProc, _tiffFileWriteProc,
+							_tiffFileReadProcW, _tiffFileWriteProc,
 							_tiffFileSeekProcW, _tiffFileCloseProc,
 							_tiffFileSizeProcW, _tiffDummyMapProc,
 							_tiffDummyUnmapProc);
 	else
-#endif
 		tif = TIFFClientOpen("TIFFMemFile", Mode,
 							NULL,
 							_tiffFileReadProc, _tiffFileWriteProc,
@@ -867,10 +872,10 @@ TIFF *iTIFFOpen(char *Mode)
 
 
 //! Writes a Tiff file
-ILboolean ilSaveTiff(ILconst_string FileName)
+ILboolean ilSaveTiff(const ILstring FileName)
 {
-	//ILHANDLE	TiffFile;
-	ILboolean	bTiff = IL_FALSE;
+	ILHANDLE	TiffFile;
+	ILuint		TiffSize;
 
 	if (ilGetBoolean(IL_FILE_MODE) == IL_FALSE) {
 		if (iFileExists(FileName)) {
@@ -878,57 +883,49 @@ ILboolean ilSaveTiff(ILconst_string FileName)
 			return IL_FALSE;
 		}
 	}
-/*
+
 	TiffFile = iopenw(FileName);
 	if (TiffFile == NULL) {
 		ilSetError(IL_COULD_NOT_OPEN_FILE);
-		return bTiff;
+		return IL_FALSE;
 	}
-*/
-	//bTiff = ilSaveTiffF(TiffFile);
-	bTiff = iSaveTiffInternal(FileName);
 
-/*
+	TiffSize = ilSaveTiffF(TiffFile);
 	iclosew(TiffFile);
-*/
-	return bTiff;
+
+	if (TiffSize == 0)
+		return IL_FALSE;
+	return IL_TRUE;
 }
 
 
 //! Writes a Tiff to an already-opened file
-ILboolean ilSaveTiffF(ILHANDLE File)
+ILuint ilSaveTiffF(ILHANDLE File)
 {
-	File;
-/*
+	ILuint Pos;
 	iSetOutputFile(File);
-	return iSaveTiffInternal();
-
-*/
-	//ilSetError(IL_FILE_READ_ERROR);
-	return IL_FALSE;
+	Pos = itellw();
+	if (iSaveTiffInternal() == IL_FALSE)
+		return 0;  // Error occurred
+	return itellw() - Pos;  // Return the number of bytes written.
 }
 
 
 //! Writes a Tiff to a memory "lump"
-ILboolean ilSaveTiffL(const void *Lump, ILuint Size)
+ILuint ilSaveTiffL(void *Lump, ILuint Size)
 {
-/*
-
+	ILuint Pos = itellw();
 	iSetOutputLump(Lump, Size);
-	return iSaveTiffInternal();
-
-*/
-	//ilSetError(IL_FILE_READ_ERROR);
-	return IL_FALSE;
+	if (iSaveTiffInternal() == IL_FALSE)
+		return 0;  // Error occurred
+	return itellw() - Pos;  // Return the number of bytes written.
 }
 
 
 // @TODO:  Accept palettes!
 
 // Internal function used to save the Tiff.
-//ILboolean iSaveTiffInternal()
-
-ILboolean iSaveTiffInternal(ILconst_string Filename)
+ILboolean iSaveTiffInternal(/*ILconst_string Filename*/)
 {
 	ILenum	Format;
 	ILenum	Compression;
@@ -972,12 +969,14 @@ ILboolean iSaveTiffInternal(ILconst_string Filename)
 		TempImage = iCurImage;
 	}
 
-	#ifndef _UNICODE
+	/*#ifndef _UNICODE
 		File = TIFFOpen(Filename, "w");
 	#else
 		File = TIFFOpenW(Filename, "w");
-	#endif
-	
+	#endif*/
+
+	// Control writing functions ourself.
+	File = iTIFFOpen("w");
 	if (File == NULL) {
 		ilSetError(IL_COULD_NOT_OPEN_FILE);
 		return IL_FALSE;
@@ -991,7 +990,7 @@ ILboolean iSaveTiffInternal(ILconst_string Filename)
 	TIFFSetField(File, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
 	TIFFSetField(File, TIFFTAG_BITSPERSAMPLE, TempImage->Bpc << 3);
 	TIFFSetField(File, TIFFTAG_SAMPLESPERPIXEL, TempImage->Bpp);
-	if(TempImage->Bpp == 4) //TODO: LUMINANCE, LUMINANCE_ALPHA
+	if (TempImage->Bpp == 4) //TODO: LUMINANCE, LUMINANCE_ALPHA
 		TIFFSetField(File, TIFFTAG_MATTEING, 1);
 	TIFFSetField(File, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
 	TIFFSetField(File, TIFFTAG_ROWSPERSTRIP, 1);
@@ -1025,26 +1024,20 @@ ILboolean iSaveTiffInternal(ILconst_string Filename)
 		ifree(str);
 	}
 
-	//The date time string is not formatted correctly,
-	//so leave it out for now
-	//(from http://www.awaresystems.be/imaging/tiff/tifftags/datetime.html :
-	//The format is: "YYYY:MM:DD HH:MM:SS", with hours like those on
-	//a 24-hour clock, and one space character between the date and the
-	//time. The length of the string, including the terminating NUL, is
-	//20 bytes.)
-	//TIFFSetField(File, TIFFTAG_DATETIME, iMakeString());
+	// Set the date and time string.
+	TIFFSetField(File, TIFFTAG_DATETIME, iMakeString());
 
 	// 24/4/2003
-	// Orientation flag is not always supported ( Photoshop, ...), orient the image data 
+	// Orientation flag is not always supported (Photoshop, ...), orient the image data 
 	// and set it always to normal view
-	TIFFSetField(File, TIFFTAG_ORIENTATION,ORIENTATION_TOPLEFT );
-	if( TempImage->Origin != IL_ORIGIN_UPPER_LEFT ) {
+	TIFFSetField(File, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
+	if (TempImage->Origin != IL_ORIGIN_UPPER_LEFT) {
 		ILubyte* Data = iGetFlipped(TempImage);
 		OldData = TempImage->Data;
 		TempImage->Data = Data;
 	}
 	else
-	OldData = TempImage->Data;
+		OldData = TempImage->Data;
 
 	/*
 	 TIFFSetField(File, TIFFTAG_ORIENTATION,
@@ -1076,7 +1069,7 @@ ILboolean iSaveTiffInternal(ILconst_string Filename)
  		ilSwapColours();
 
 	if (TempImage->Data != OldData) {
-		ifree( TempImage->Data );
+		ifree(TempImage->Data);
 		TempImage->Data = OldData;
 	}
 
@@ -1090,13 +1083,18 @@ ILboolean iSaveTiffInternal(ILconst_string Filename)
 
 /*----------------------------------------------------------------------------*/
 // Makes a neat date string for the date field.
+// From http://www.awaresystems.be/imaging/tiff/tifftags/datetime.html :
+// The format is: "YYYY:MM:DD HH:MM:SS", with hours like those on
+// a 24-hour clock, and one space character between the date and the
+// time. The length of the string, including the terminating NUL, is
+// 20 bytes.)
 char *iMakeString()
 {
-	static char TimeStr[255];
+	static char TimeStr[20];
 	time_t		Time;
 	struct tm	*CurTime;
 
-	imemclear(TimeStr, 255);
+	imemclear(TimeStr, 20);
 
 	time(&Time);
 #ifdef _WIN32
@@ -1104,8 +1102,8 @@ char *iMakeString()
 #endif
 	CurTime = localtime(&Time);
 
-	strftime(TimeStr, 255, "%s (%z)", CurTime); // "%#c (%z)"	// %s was %c
-
+	strftime(TimeStr, 20, "%Y:%m:%d %H:%M:%S", CurTime);
+	
 	return TimeStr;
 }
 
